/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2009
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	 main.c
**  Abstract :	This file implements main function.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0515_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2009/10/19
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Int.h"
#include "Serial.h"
#include "Port.h"
#include "System.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function  implements main function.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
unsigned long	mhz_no;
unsigned int	set_mhz;
unsigned int	direct[20];
unsigned char	preset_no;
unsigned char	stereo;
unsigned char	mode;
unsigned char	ant;
unsigned char	preset_no_max;

unsigned char	eeprom_addr=0xa0;
unsigned char	eeprom_finish;
unsigned char	eeprom_write_data[3];
unsigned char	eeprom_read_data[1];

unsigned int	mode_cnt;


void	init_data(void)
{
	mhz_no=864;
	stereo=1;
	preset_no=0;
	preset_no_max=0;
	mode=0;
	ant=0;
}

void	wait_time(unsigned int time)
{
	while(time!=0){
		time--;
	}
}

/*----------  LCD MODULE ----------*/
void	lcd_db(unsigned char data)
{
	P2.0=data&0x01;
	P2.1=(data&0x02)>>1;
	P2.2=(data&0x04)>>2;
	P2.3=(data&0x08)>>3;
}
void	lcd_wait(void)
{
	wait_time(5000);
}
void	lcd_c_set(void)
{
	P13.0=0;
	P14.0=1;
	wait_time(20);
	P14.0=0;
}
void	lcd_d_set(void)
{
	P13.0=1;
	P14.0=1;
	wait_time(20);
	P14.0=0;
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(500);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(500);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_mhz(unsigned int hz)
{
unsigned char n0,n1,n2,n3;
unsigned int count;

	lcd_xy(0,0);
	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	if(n3==0){lcd_print(" ");}
	else{lcd_su(n3);}
	lcd_su(n2);
	lcd_su(n1);
	lcd_print(".");
	lcd_su(n0);
}
void	lcd_ini_disp(void)
{
	lcd_xy(0,0);
	lcd_print("   . M");
}
void	lcd_stereo(void)
{
	lcd_xy(7,0);
	switch(stereo){
		case 1:lcd_print("S");break;
		case 0:lcd_print("M");break;
	}
}
void	lcd_ant(void)
{
	lcd_xy(0,1);
	switch(ant){
		case 0:lcd_print("A");break;
		case 1:lcd_print("B");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_4keta10(unsigned int count)
{
unsigned char n0,n1,n2,n3;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	presetmei(unsigned char no)
{
	lcd_xy(3,1);
	lcd_print("CH-  ");
	lcd_xy(6,1);lcd_2keta10(no);
}
void	tunemei(void)
{
	lcd_xy(3,1);
	lcd_print("TUNE ");
}
void	setmei(unsigned char no)
{
	lcd_xy(3,1);
	lcd_print("SET  ");
	lcd_xy(6,1);lcd_2keta10(no);
}
void	lcd_ok(void)
{
	lcd_xy(3,1);
	lcd_print("SETOK");
}
void	lcd_del(void)
{
	lcd_xy(3,1);
	lcd_print("-DEL-");
}

/*----------  MHZ    ----------*/
void	mhz_inc(void)
{
	mhz_no++;
	if(mhz_no==1081){mhz_no=760;}
}

void	mhz_dec(void)
{
	mhz_no--;
	if(mhz_no==759){mhz_no=1080;}
}
void	mhz_inc_10(void)
{
	mhz_no=mhz_no/10;
	mhz_no=mhz_no*10+10;
	if(mhz_no>=1081){mhz_no=760;}
}

void	mhz_dec_10(void)
{
	mhz_no=mhz_no/10;
	mhz_no=mhz_no*10-10;
	if(mhz_no<=759){mhz_no=1080;}
}
/*---------- PRESET ---------*/
unsigned int preset_mhz(unsigned char no)
{
	if(no>=20){no=19;}
	return direct[no];
}
void	preset_up_mode1(void)
{
	if(preset_no==preset_no_max){preset_no=0;}
	else{preset_no++;}
}
void	preset_down_mode1(void)
{
	if(preset_no<=0){preset_no=preset_no_max;}
	else{preset_no--;}
}
void	preset_up_mode2(void)
{
	if(preset_no_max<19){
		if(preset_no==preset_no_max+1){preset_no=0;}
		else{preset_no++;}
	}
	else{
		preset_no++;
		if(preset_no>=20){preset_no=0;}
		}
}
void	preset_down_mode2(void)
{
	if(preset_no_max<19){
		if(preset_no<=0){preset_no=preset_no_max+1;}
		else{preset_no--;}
	}
	else{
		if(preset_no<=0){preset_no=19;}
		else{preset_no--;}
	}
}

/*---------- FM CHIP ---------*/
void	data_out(unsigned char ad,unsigned char da)
{
unsigned char	i;

	P3.0=0;
	P3.1=0;
	P3.2=0;

	for(i=0;i<4;i++){
		if((ad&0x01)==0x01){
			P3.1=1;
		}
		else{
			P3.1=0;
		}
		ad=ad>>1;
		P3.0=1;
		wait_time(30);
		P3.0=0;
	}
	
	for(i=0;i<8;i++){
		if((da&0x01)==0x01){
			P3.1=1;
		}
		else{
			P3.1=0;
		}
		da=da>>1;
		P3.0=1;
		wait_time(30);
		P3.0=0;
	}
	P3.2=1;
	wait_time(30);
	P3.2=0;
	
	P3.1=0;
}

void	fm_chip_init(void)
{
	data_out(0x00,0x10);
	data_out(0x01,0x0f);
	data_out(0x02,0xe4);
	data_out(0x03,0x37);
	data_out(0x04,0xd8);
	data_out(0x05,0x5d);
	data_out(0x06,0x88);
	data_out(0x07,0x4c);
	data_out(0x08,0x09);
	data_out(0x09,0x4d);
	data_out(0x0a,0x28);
	data_out(0x0b,0x8c);
	data_out(0x0c,0x04);
	data_out(0x0d,0x00);
}
void	fm_data_set(unsigned long data)
{
unsigned long	dd;
unsigned int	d1,d2;
unsigned char	d3,d4;

	dd=(data*100-375)*1000/8192;
	
	d1=(unsigned int)dd%256;
	
	dd=(dd-(unsigned long)d1)/256;
	d2=(unsigned int)dd%256;
	
	d3=(unsigned char)d1;
	d4=(unsigned char)d2|0xc0;
	
	data_out(0x01,d3);
	data_out(0x02,d4);
		
	if((mhz_no>=760)&&(mhz_no<940)){data_out(0x07,0x4c);}
	if((mhz_no>=940)&&(mhz_no<1000)){data_out(0x07,0x4b);}
	if((mhz_no>=1000)&&(mhz_no<1081)){data_out(0x07,0x4a);}

}
void	mute(unsigned char f)
{
	switch(stereo){
		case 1:	switch(f){
					case 1:data_out(0x00,0x13);break;
					case 0:data_out(0x00,0x11);break;
				}
				break;
		case 0:	switch(f){
					case 1:data_out(0x00,0x1b);break;
					case 0:data_out(0x00,0x19);break;
				}
				break;
	}
}

/*---------- EEPROM ---------*/
void	eeprom_write(unsigned char ad,unsigned char d)
{
	eeprom_write_data[0]=0;
	eeprom_write_data[1]=ad;
	eeprom_write_data[2]=d;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 3,1);
	while (!eeprom_finish){};
}
unsigned char	eeprom_read(unsigned char ad)
{
unsigned char	data;

	eeprom_write_data[0]=0;
	eeprom_write_data[1]=ad;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 2,1);
	while (!eeprom_finish){};

	eeprom_finish = 0 ;
	IIC0_MasterReceiveStart(eeprom_addr, eeprom_read_data,1,10);
	while (!eeprom_finish) {};
	
	data=eeprom_read_data[0];
	
	return data;
}
void	eeprom_write_sub(void)
{
		eeprom_write(0x10,(unsigned char)(direct[0]%256));wait_time(1000);
		eeprom_write(0x11,(unsigned char)(direct[0]/256));wait_time(1000);
		eeprom_write(0x12,(unsigned char)(direct[1]%256));wait_time(1000);
		eeprom_write(0x13,(unsigned char)(direct[1]/256));wait_time(1000);
		eeprom_write(0x14,(unsigned char)(direct[2]%256));wait_time(1000);
		eeprom_write(0x15,(unsigned char)(direct[2]/256));wait_time(1000);
		eeprom_write(0x16,(unsigned char)(direct[3]%256));wait_time(1000);
		eeprom_write(0x17,(unsigned char)(direct[3]/256));wait_time(1000);
		eeprom_write(0x18,(unsigned char)(direct[4]%256));wait_time(1000);
		eeprom_write(0x19,(unsigned char)(direct[4]/256));wait_time(1000);
		eeprom_write(0x1a,(unsigned char)(direct[5]%256));wait_time(1000);
		eeprom_write(0x1b,(unsigned char)(direct[5]/256));wait_time(1000);
		eeprom_write(0x1c,(unsigned char)(direct[6]%256));wait_time(1000);
		eeprom_write(0x1d,(unsigned char)(direct[6]/256));wait_time(1000);
		eeprom_write(0x1e,(unsigned char)(direct[7]%256));wait_time(1000);
		eeprom_write(0x1f,(unsigned char)(direct[7]/256));wait_time(1000);
		eeprom_write(0x20,(unsigned char)(direct[8]%256));wait_time(1000);
		eeprom_write(0x21,(unsigned char)(direct[8]/256));wait_time(1000);
		eeprom_write(0x22,(unsigned char)(direct[9]%256));wait_time(1000);
		eeprom_write(0x23,(unsigned char)(direct[9]/256));wait_time(1000);
		eeprom_write(0x24,(unsigned char)(direct[10]%256));wait_time(1000);
		eeprom_write(0x25,(unsigned char)(direct[10]/256));wait_time(1000);
		eeprom_write(0x26,(unsigned char)(direct[11]%256));wait_time(1000);
		eeprom_write(0x27,(unsigned char)(direct[11]/256));wait_time(1000);
		eeprom_write(0x28,(unsigned char)(direct[12]%256));wait_time(1000);
		eeprom_write(0x29,(unsigned char)(direct[12]/256));wait_time(1000);
		eeprom_write(0x2a,(unsigned char)(direct[13]%256));wait_time(1000);
		eeprom_write(0x2b,(unsigned char)(direct[13]/256));wait_time(1000);
		eeprom_write(0x2c,(unsigned char)(direct[14]%256));wait_time(1000);
		eeprom_write(0x2d,(unsigned char)(direct[14]/256));wait_time(1000);
		eeprom_write(0x2e,(unsigned char)(direct[15]%256));wait_time(1000);
		eeprom_write(0x2f,(unsigned char)(direct[15]/256));wait_time(1000);
		eeprom_write(0x30,(unsigned char)(direct[16]%256));wait_time(1000);
		eeprom_write(0x31,(unsigned char)(direct[16]/256));wait_time(1000);
		eeprom_write(0x32,(unsigned char)(direct[17]%256));wait_time(1000);
		eeprom_write(0x33,(unsigned char)(direct[17]/256));wait_time(1000);
		eeprom_write(0x34,(unsigned char)(direct[18]%256));wait_time(1000);
		eeprom_write(0x35,(unsigned char)(direct[18]/256));wait_time(1000);
		eeprom_write(0x36,(unsigned char)(direct[19]%256));wait_time(1000);
		eeprom_write(0x37,(unsigned char)(direct[19]/256));wait_time(1000);
}

void	eeprom_init(void)
{
		direct[0]=864;direct[1]=0;
		direct[2]=0;direct[3]=0;
		direct[4]=0;direct[5]=0;
		direct[6]=0;direct[7]=0;
		direct[8]=0;direct[9]=0;
		direct[10]=0;direct[11]=0;
		direct[12]=0;direct[13]=0;
		direct[14]=0;direct[15]=0;
		direct[16]=0;direct[17]=0;
		direct[18]=0;direct[19]=0;

		eeprom_write(0,0xa5);wait_time(1000);
		eeprom_write_sub();

		eeprom_write(0x0a,(unsigned char)(mhz_no%256));wait_time(1000);
		eeprom_write(0x0b,(unsigned char)(mhz_no/256));wait_time(1000);
		eeprom_write(0x08,preset_no);wait_time(1000);
		eeprom_write(0x05,mode);wait_time(1000);
		eeprom_write(0x06,stereo);wait_time(1000);
		eeprom_write(0x07,ant);wait_time(1000);
		eeprom_write(0x09,preset_no_max);wait_time(1000);
}

void	eeprom_check(void)
{
unsigned int sub0,sub1;

	if(eeprom_read(0)==0xa5){
		sub0=(unsigned int)eeprom_read(0x10);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x11);wait_time(1000);
		direct[0]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x12);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x13);wait_time(1000);
		direct[1]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x14);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x15);wait_time(1000);
		direct[2]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x16);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x17);wait_time(1000);
		direct[3]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x18);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x19);wait_time(1000);
		direct[4]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x1a);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x1b);wait_time(1000);
		direct[5]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x1c);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x1d);wait_time(1000);
		direct[6]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x1e);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x1f);wait_time(1000);
		direct[7]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x20);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x21);wait_time(1000);
		direct[8]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x22);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x23);wait_time(1000);
		direct[9]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x24);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x25);wait_time(1000);
		direct[10]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x26);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x27);wait_time(1000);
		direct[11]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x28);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x29);wait_time(1000);
		direct[12]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x2a);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x2b);wait_time(1000);
		direct[13]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x2c);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x2d);wait_time(1000);
		direct[14]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x2e);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x2f);wait_time(1000);
		direct[15]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x30);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x31);wait_time(1000);
		direct[16]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x32);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x33);wait_time(1000);
		direct[17]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x34);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x35);wait_time(1000);
		direct[18]=sub0+sub1*256;
		sub0=(unsigned int)eeprom_read(0x36);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x37);wait_time(1000);
		direct[19]=sub0+sub1*256;

		sub0=(unsigned int)eeprom_read(0x0a);wait_time(1000);
		sub1=(unsigned int)eeprom_read(0x0b);wait_time(1000);
		mhz_no=sub0+sub1*256;
		preset_no=eeprom_read(0x08);wait_time(1000);
		mode=eeprom_read(0x05);wait_time(1000);
		stereo=eeprom_read(0x06);wait_time(1000);
		ant=eeprom_read(0x07);wait_time(1000);
		preset_no_max=eeprom_read(0x09);wait_time(1000);
	}
	else{
		eeprom_init();
	}
}
void	write_direct(void){

		eeprom_write(0x0a,(unsigned char)(mhz_no%256));wait_time(1000);
		eeprom_write(0x0b,(unsigned char)(mhz_no/256));wait_time(1000);
		eeprom_write(0x08,preset_no);wait_time(1000);
		eeprom_write(0x05,mode);wait_time(1000);
		eeprom_write(0x06,stereo);wait_time(1000);
		eeprom_write(0x07,ant);wait_time(1000);
		eeprom_write(0x09,preset_no_max);wait_time(1000);
}
void	write_direct_set(unsigned char	no){

		eeprom_write(0x10+no*2,(unsigned char)(direct[no]%256));wait_time(1000);
		eeprom_write(0x11+no*2,(unsigned char)(direct[no]/256));wait_time(1000);
}

/*----------- MODE -----------*/
void	shift_mode0(void)
{
		mode=0;
		lcd_mhz((unsigned int)mhz_no);
		tunemei();
		fm_data_set(mhz_no);
		if(preset_no>preset_no_max){
			preset_no=preset_no_max;
		}
}

/*---------- DEL ------------*/
void	direct_del(void)
{
unsigned int	i;

	if(preset_no>preset_no_max){
		return;
	}
	
	for(i=preset_no+1;i<20;i++){
		direct[i-1]=direct[i];
	}
	lcd_del();
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);
	preset_no_max--;
	preset_no=0;
}

/*---------- MAIN -----------*/
void  main( void )
{

unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;
unsigned char	sw5_key;
unsigned char	sw5_rep;
unsigned char	sw5_cnt;


	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;
	sw5_key=0;sw5_rep=0;sw5_cnt=0;
	preset_no=0;

	P3.0=0;
	P3.1=0;
	P3.2=0;
	P0.1=0;
	
	mode_cnt=0;

	wait_time(10000);

	init_data();

	lcd_init();
	lcd_clear();
	lcd_ini_disp();

	if((P7.0==0)&(P7.1==0)){
		eeprom_init();
	}
	else{
		eeprom_check();
	}
	fm_chip_init();

	switch(mode){
		case 0:	lcd_mhz((unsigned int)mhz_no);
				tunemei();
				fm_data_set(mhz_no);
				break;
		case 1:	mhz_no=preset_mhz(preset_no);
				presetmei(preset_no);
				lcd_mhz((unsigned int)mhz_no);
				fm_data_set(mhz_no);
				break;
	}
	switch(stereo){
		case 0:lcd_stereo();data_out(0x00,0x19);break;
		case 1:lcd_stereo();data_out(0x00,0x11);break;
	}
	switch(ant){
		case 0:lcd_ant();P0.1=0;break;
		case 1:lcd_ant();P0.1=1;break;
	}

	/* Start user code. Do not edit comment generated here */
	while (1) {
	/* key scan 1 */
		if (P7.0==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (P7.1==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (P7.2==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if (P7.3==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/
		if (P1.0==0){
				sw5_key=0;
		}
		else{
			sw5_key=1;
			sw5_rep=0;
			sw5_cnt=0;
		}
		/*------------------*/

		wait_time(2500);



	/* key scan 2 */

		if ((P7.0==0)&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((P7.1==0)&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((P7.2==0)&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if ((P7.3==0)&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/
		if ((P1.0==0)&(sw5_key==0)){
			sw5_key=0;
		}
		else{
			sw5_key=1;
			sw5_rep=0;
			sw5_cnt=0;
		}
		/*------------------*/


	/* key action */
		switch(mode){
			case 0:
				if((sw1_key==0)&(sw1_rep==0)){
					sw1_rep=1;
					mute(1);
					mhz_inc();
					lcd_mhz((unsigned int)mhz_no);
					tunemei();
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				if((sw1_key==0)&(sw1_rep==1)){
					sw1_cnt++;
					if(sw1_cnt>=35){
						mute(1);
						mhz_inc_10();
						lcd_mhz((unsigned int)mhz_no);
						tunemei();
						fm_data_set(mhz_no);
						write_direct();
						mute(0);
						sw1_cnt=0;
					}
				}
				/*------------------*/
				if((sw2_key==0)&(sw2_rep==0)){
					sw2_rep=1;
					mute(1);
					mhz_dec();
					lcd_mhz((unsigned int)mhz_no);
					tunemei();
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				if((sw2_key==0)&(sw2_rep==1)){
					sw2_cnt++;
					if(sw2_cnt>=35){
						mute(1);
						mhz_dec_10();
						lcd_mhz((unsigned int)mhz_no);
						tunemei();
						fm_data_set(mhz_no);
						write_direct();
						mute(0);
						sw2_cnt=0;
					}
				}
				/*------------------*/
				if((sw3_key==0)&(sw3_rep==0)){
					sw3_rep=1;
					mute(1);
					mode=1;
					mhz_no=preset_mhz(preset_no);
					presetmei(preset_no);
					lcd_mhz((unsigned int)mhz_no);
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				/*------------------*/
				if((sw4_key==0)&(sw4_rep==0)){
					sw4_rep=1;
					switch(stereo){
						case 1:stereo=0;lcd_stereo();data_out(0x00,0x19);break;
						case 0:stereo=1;lcd_stereo();data_out(0x00,0x11);break;
					}
					write_direct();
				}
				if((sw4_key==0)&(sw4_rep==1)){
					sw4_cnt++;
					if(sw4_cnt>=50){
						switch(ant){
							case 0:ant=1;lcd_ant();P0.1=1;break;
							case 1:ant=0;lcd_ant();P0.1=0;break;
						}
						write_direct();
						sw4_cnt=0;
					}
				}
				/*------------------*/
				if((sw5_key==0)&(sw5_rep==0)){
					sw5_rep=1;
				}
				if((sw5_key==0)&(sw5_rep==1)){
					sw5_cnt++;
					if(sw5_cnt>=100){
						mode=2;
						setmei(preset_no);
						mode_cnt=800;
						sw5_cnt=0;
					}
				}
				/*------------------*/
				break;
			case 1:
				if((sw1_key==0)&(sw1_rep==0)){
					sw1_rep=1;
					mute(1);
					preset_up_mode1();
					mhz_no=preset_mhz(preset_no);
					presetmei(preset_no);
					lcd_mhz((unsigned int)mhz_no);
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				/*--------------------*/
				if((sw2_key==0)&(sw2_rep==0)){
					sw2_rep=1;
					mute(1);
					preset_down_mode1();
					mhz_no=preset_mhz(preset_no);
					presetmei(preset_no);
					lcd_mhz((unsigned int)mhz_no);
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				/*------------------*/
				if((sw3_key==0)&(sw3_rep==0)){
					sw3_rep=1;
					mute(1);
					mode=0;
					lcd_mhz((unsigned int)mhz_no);
					tunemei();
					fm_data_set(mhz_no);
					write_direct();
					mute(0);
				}
				/*------------------*/
				if((sw4_key==0)&(sw4_rep==0)){
					sw4_rep=1;
					switch(stereo){
						case 1:stereo=0;lcd_stereo();data_out(0x00,0x19);break;
						case 0:stereo=1;lcd_stereo();data_out(0x00,0x11);break;
					}
					write_direct();
				}
				if((sw4_key==0)&(sw4_rep==1)){
					sw4_cnt++;
					if(sw4_cnt>=50){
						switch(ant){
							case 0:ant=1;lcd_ant();P0.1=1;break;
							case 1:ant=0;lcd_ant();P0.1=0;break;
						}
						write_direct();
						sw4_cnt=0;
					}
				}
				/*------------------*/
				if((sw5_key==0)&(sw5_rep==0)){
					sw5_rep=1;
				}
				if((sw5_key==0)&(sw5_rep==1)){
					sw5_cnt++;
					if(sw5_cnt>=100){
						mode=2;
						setmei(preset_no);
						mode_cnt=800;
						sw5_cnt=0;
					}
				}
				/*------------------*/
				break;
			case 2:
				if((sw1_key==0)&(sw1_rep==0)){
					sw1_rep=1;
					preset_up_mode2();
					setmei(preset_no);
					mode_cnt=800;
				}
				/*--------------------*/
				if((sw2_key==0)&(sw2_rep==0)){
					sw2_rep=1;
					preset_down_mode2();
					setmei(preset_no);
					mode_cnt=800;
				}
				/*------------------*/
				if((sw3_key==0)&(sw3_rep==0)){
					sw3_rep=1;
					direct[preset_no]=(unsigned int)mhz_no;
					write_direct_set(preset_no);
					lcd_ok();
					wait_time(50000);
					wait_time(50000);
					wait_time(50000);
					setmei(preset_no);
					if(preset_no>preset_no_max){
						preset_no_max=preset_no;
					}
					shift_mode0();
					write_direct();
					mode_cnt=0;
				}
				/*------------------*/
				if((sw4_key==0)&(sw4_rep==0)){
					sw4_rep=1;
					direct_del();
					shift_mode0();
					eeprom_write_sub();
					write_direct();
					mode_cnt=0;
				}
				/*------------------*/
				if((sw5_key==0)&(sw5_rep==0)){
					sw5_rep=1;
					mode_cnt=800;
				}
				if((sw5_key==0)&(sw5_rep==1)){
					sw5_cnt++;
					if(sw5_cnt>=100){
						shift_mode0();
						write_direct();
						mode_cnt=0;
						sw5_cnt=0;
					}
				}
				/*------------------*/
				break;
		}

	/* key scan 3 */
		if (P7.0==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (P7.1==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (P7.2==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if (P7.3==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/
		if (P1.0==0){
				sw5_key=0;
		}
		else{
			sw5_key=1;
			sw5_rep=0;
			sw5_cnt=0;
		}
		/*------------------*/

		if((mode==0)||(mode==1)){
			if((P7.0==1)&(P7.1==1)&(P7.2==1)&(P7.3==1)&(P1.0==1)){
				STOP();
			}
		}

		if(mode==2){
			if(mode_cnt>0){
				mode_cnt--;
				if(mode_cnt==0){
					mode=0;
					shift_mode0();
					write_direct();
					mode_cnt=0;
				}
			}
		}

	}
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */



